@@ -34,6 +34,11 @@ use super::search::format_duration_into;
 pub enum Cmd {
     /// Begins a new command in the history
     Start {
+        /// Collects the command from the `ATUIN_COMMAND_LINE` environment variable,
+        /// which does not need escaping and is more compatible between OS and shells
+        #[arg(long = "command-from-env", hide = true)]
+        cmd_env: bool,
+
         command: Vec<String>,
     },
 
@@ -80,7 +85,7 @@ pub enum Cmd {
         #[arg(long, visible_alias = "tz")]
         timezone: Option<Timezone>,
 
-        /// Available variables: {command}, {directory}, {duration}, {user}, {host}, {exit} and {time}.
+        /// Available variables: {command}, {directory}, {duration}, {user}, {host}, {exit}, {time}, {session}, and {uuid}
         /// Example: --format "{time} - [{duration}] - {directory}$\t{command}"
         #[arg(long, short)]
         format: Option<String>,
@@ -103,7 +108,7 @@ pub enum Cmd {
         #[arg(long, visible_alias = "tz")]
         timezone: Option<Timezone>,
 
-        /// Available variables: {command}, {directory}, {duration}, {user}, {host} and {time}.
+        /// Available variables: {command}, {directory}, {duration}, {user}, {host}, {time}, {session}, {uuid} and {relativetime}.
         /// Example: --format "{time} - [{duration}] - {directory}$\t{command}"
         #[arg(long, short)]
         format: Option<String>,
@@ -315,6 +320,8 @@ impl FormatKey for FmtHistory<'_> {
                     .split_once(':')
                     .map_or("", |(_, user)| user),
             )?,
+            "session" => f.write_str(&self.history.session)?,
+            "uuid" => f.write_str(&self.history.id.0)?,
             _ => return Err(FormatKeyError::UnknownKey),
         }
         Ok(())
@@ -344,13 +351,7 @@ fn parse_fmt(format: &str) -> ParsedFmt<'_> {
 
 impl Cmd {
     #[allow(clippy::too_many_lines, clippy::cast_possible_truncation)]
-    async fn handle_start(
-        db: &impl Database,
-        settings: &Settings,
-        command: &[String],
-    ) -> Result<()> {
-        let command = command.join(" ");
-
+    async fn handle_start(db: &impl Database, settings: &Settings, command: &str) -> Result<()> {
         // It's better for atuin to silently fail here and attempt to
         // store whatever is ran, than to throw an error to the terminal
         let cwd = utils::get_current_dir();
@@ -375,9 +376,7 @@ impl Cmd {
     }
 
     #[cfg(feature = "daemon")]
-    async fn handle_daemon_start(settings: &Settings, command: &[String]) -> Result<()> {
-        let command = command.join(" ");
-
+    async fn handle_daemon_start(settings: &Settings, command: &str) -> Result<()> {
         // It's better for atuin to silently fail here and attempt to
         // store whatever is ran, than to throw an error to the terminal
         let cwd = utils::get_current_dir();
@@ -517,7 +516,10 @@ impl Cmd {
             (true, true) => [Session, Directory],
             (true, false) => [Session, Global],
             (false, true) => [Global, Directory],
-            (false, false) => [settings.default_filter_mode(), Global],
+            (false, false) => [
+                settings.default_filter_mode(context.git_root.is_some()),
+                Global,
+            ],
         };
 
         let history = db
@@ -655,7 +657,8 @@ impl Cmd {
         // Skip initializing any databases for start/end, if the daemon is enabled
         if settings.daemon.enabled {
             match self {
-                Self::Start { command } => {
+                Self::Start { .. } => {
+                    let command = self.get_start_command().unwrap_or_default();
                     return Self::handle_daemon_start(settings, &command).await;
                 }
 
@@ -681,7 +684,10 @@ impl Cmd {
         let history_store = HistoryStore::new(store.clone(), host_id, encryption_key);
 
         match self {
-            Self::Start { command } => Self::handle_start(&db, settings, &command).await,
+            Self::Start { .. } => {
+                let command = self.get_start_command().unwrap_or_default();
+                Self::handle_start(&db, settings, &command).await
+            }
             Self::End { id, exit, duration } => {
                 Self::handle_end(&db, store, history_store, settings, &id, exit, duration).await
             }
@@ -750,6 +756,18 @@ impl Cmd {
             }
         }
     }
+
+    /// Returns the command line to use for the `Start` variant.
+    /// Returns `None` for any other variant.
+    fn get_start_command(&self) -> Option<String> {
+        match self {
+            Self::Start { cmd_env: true, .. } => {
+                Some(std::env::var("ATUIN_COMMAND_LINE").unwrap_or_default())
+            }
+            Self::Start { command, .. } => Some(command.join(" ")),
+            _ => None,
+        }
+    }
 }
 
 #[cfg(test)]
